/** @file wxw_input.h

	Skelton for retropc emulator
	wxWidgets edition

	@author Sasaji
	@date   2012.02.21

	@brief [ wxw input ]

*/

#ifndef WXW_INPUT_H
#define WXW_INPUT_H

#include "../../emu.h"
#include "../../vm/vm.h"
#include "../../keycode.h"

#if defined(__WXMSW__)

typedef struct st_nc2kc {
	short nc;
	short kc;
} nc2kc_t;

/// mapping from VK_* virtual key to original keycode
const uint8_t vkkey2keycode[256] = {
	/* 0x00 - 0x0f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	KEYCODE_BACKSPACE,
	KEYCODE_TAB,
	0,
	0,
	KEYCODE_CLEAR,
	KEYCODE_RETURN,
	0,
	0,
	/* 0x10 - 0x1f */
	KEYCODE_SHIFT,
	KEYCODE_CONTROL,
	KEYCODE_ALT,
	KEYCODE_PAUSE,
	KEYCODE_CAPSLOCK,
	KEYCODE_KATAHIRA,
	0,
	0,
	0,
	KEYCODE_GRAVE,
	0,
	KEYCODE_ESCAPE,
	KEYCODE_HENKAN,
	KEYCODE_MUHENKAN,
	0,
	0,
	/* 0x20 - 0x2f */
	KEYCODE_SPACE,
	KEYCODE_PAGEUP,
	KEYCODE_PAGEDOWN,
	KEYCODE_END,
	KEYCODE_HOME,
	KEYCODE_LEFT,
	KEYCODE_UP,
	KEYCODE_RIGHT,
	KEYCODE_DOWN,
	KEYCODE_SELECT,
	KEYCODE_PRINT,
	0,
	0,
	KEYCODE_INSERT,
	KEYCODE_DELETE,
	KEYCODE_HELP,
	/* 0x30 - 0x3f */
	KEYCODE_0,
	KEYCODE_1,
	KEYCODE_2,
	KEYCODE_3,
	KEYCODE_4,
	KEYCODE_5,
	KEYCODE_6,
	KEYCODE_7,
	KEYCODE_8,
	KEYCODE_9,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x40 - 0x4f */
	0,
	KEYCODE_A,
	KEYCODE_B,
	KEYCODE_C,
	KEYCODE_D,
	KEYCODE_E,
	KEYCODE_F,
	KEYCODE_G,
	KEYCODE_H,
	KEYCODE_I,
	KEYCODE_J,
	KEYCODE_K,
	KEYCODE_L,
	KEYCODE_M,
	KEYCODE_N,
	KEYCODE_O,
	/* 0x50 - 0x5f */
	KEYCODE_P,
	KEYCODE_Q,
	KEYCODE_R,
	KEYCODE_S,
	KEYCODE_T,
	KEYCODE_U,
	KEYCODE_V,
	KEYCODE_W,
	KEYCODE_X,
	KEYCODE_Y,
	KEYCODE_Z,
	KEYCODE_LSUPER,
	KEYCODE_RSUPER,
	KEYCODE_MENU,
	0,
	0, // KEYCODE_SLEEP
	/* 0x60 - 0x6f */
	KEYCODE_KP_0,
	KEYCODE_KP_1,
	KEYCODE_KP_2,
	KEYCODE_KP_3,
	KEYCODE_KP_4,
	KEYCODE_KP_5,
	KEYCODE_KP_6,
	KEYCODE_KP_7,
	KEYCODE_KP_8,
	KEYCODE_KP_9,
	KEYCODE_KP_MULTIPLY,
	KEYCODE_KP_PLUS,
	0,
	KEYCODE_KP_MINUS,
	KEYCODE_KP_PERIOD,
	KEYCODE_KP_DIVIDE,
	/* 0x70 - 0x7f */
	KEYCODE_F1,
	KEYCODE_F2,
	KEYCODE_F3,
	KEYCODE_F4,
	KEYCODE_F5,
	KEYCODE_F6,
	KEYCODE_F7,
	KEYCODE_F8,
	KEYCODE_F9,
	KEYCODE_F10, 
	KEYCODE_F11,
	KEYCODE_F12,
	KEYCODE_F13,
	KEYCODE_F14,
	KEYCODE_F15,
	KEYCODE_F16,
	/* 0x80 - 0x8f */
	KEYCODE_F17,
	KEYCODE_F18,
	KEYCODE_F19,
	KEYCODE_F20,
	KEYCODE_F21,
	KEYCODE_F22,
	KEYCODE_WORLD_0,
	KEYCODE_WORLD_1,
	KEYCODE_WORLD_2,
	KEYCODE_WORLD_3,
	KEYCODE_WORLD_4,
	KEYCODE_WORLD_5,
	KEYCODE_WORLD_6,
	KEYCODE_WORLD_7,
	KEYCODE_WORLD_8,
	KEYCODE_WORLD_9,
	/* 0x90 - 0x9f */
	KEYCODE_NUMLOCK,
	KEYCODE_SCROLLLOCK,
	KEYCODE_KP_EQUALS,
	KEYCODE_JISHO,
	KEYCODE_MASSHOU,
	KEYCODE_TOUROKU,
	KEYCODE_LOYAYUBI,
	KEYCODE_ROYAYUBI,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xa0 - 0xaf */
	KEYCODE_LSHIFT,
	KEYCODE_RSHIFT,
	KEYCODE_LCTRL,
	KEYCODE_RCTRL,
	KEYCODE_LALT,
	KEYCODE_RALT,
	KEYCODE_BROWSER_BACK,
	KEYCODE_BROWSER_FORWARD,
	KEYCODE_BROWSER_REFRESH,
	KEYCODE_BROWSER_STOP,
	KEYCODE_BROWSER_SEARCH,
	KEYCODE_BROWSER_FAVOR, 
	KEYCODE_BROWSER_HOME, 
	KEYCODE_VOLUME_MUTE, 
	KEYCODE_VOLUME_DOWN, 
	KEYCODE_VOLUME_UP,
	/* 0xb0 - 0xbf */
	0,
	0,
	0,
	0, 
	0,
	0, 
	0, 
	0,
	0,
	0, 
	KEYCODE_COLON,
	KEYCODE_SEMICOLON,
	KEYCODE_COMMA,
	KEYCODE_MINUS, 
	KEYCODE_PERIOD, 
	KEYCODE_SLASH,
	/* 0xc0 - 0xcf */
	KEYCODE_AT,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0, 
	0,
	0,
	0, 
	0,
	0,
	0,
	0,
	/* 0xd0 - 0xdf */
	0,
	0,
	0, 
	0,
	0,
	0, 
	0, 
	0,
	0,
	0,
	0,
	KEYCODE_LBRACKET,
	KEYCODE_BACKSLASH,
	KEYCODE_RBRACKET,
	KEYCODE_CARET,
	0,
	/* 0xe0 - 0xef */
	0,
	0,
	KEYCODE_UNDERSCORE,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0, 
	0, 
	0,
	0,
	0,
	0,
	/* 0xf0 - 0xff */
	KEYCODE_CAPSLOCK,
	KEYCODE_MUHENKAN,
	KEYCODE_KATAHIRA,
	KEYCODE_HENKAN,
	KEYCODE_HENKAN,
	0, 
	0,
	0,
	0,
	0,
	0,
	0,
	0, 
	0, 
	0,
	0
};

static const uint8_t scancode2keycode47[16] = {
	/* 0x47 - 0x56 */
	KEYCODE_KP_7,
	KEYCODE_KP_8,
	KEYCODE_KP_9,
	0,
	KEYCODE_KP_4,
	KEYCODE_KP_5,
	KEYCODE_KP_6,
	0,
	KEYCODE_KP_1,
	KEYCODE_KP_2,
	KEYCODE_KP_3,
	KEYCODE_KP_0,
	KEYCODE_KP_PERIOD,
	0,
	0,
	0
};

static const uint8_t scancode2keycode70[16] = {
	/* 0x70 - 0x7f */
	KEYCODE_KATAHIRA,	// Katakana / Hiragana (JP)
	0,
	0,
	KEYCODE_UNDERSCORE,	// _
	0,
	0,
	0,
	0,
	0,
	KEYCODE_HENKAN,		// Henkan (JP)
	0,
	KEYCODE_MUHENKAN,	// Muhenkan (JP)
	0,
	KEYCODE_BACKSLASH,	// Yen
	0,
	0
};

#elif defined(__WXOSX__)

const uint8_t scancode2keycode[128] = {
	/* 0x00 - 0x0f */
	KEYCODE_A,			// A
	KEYCODE_S,			// S
	KEYCODE_D,			// D
	KEYCODE_F,			// F
	KEYCODE_H,			// H
	KEYCODE_G,			// G
	KEYCODE_Z,			// Z
	KEYCODE_X,			// X
	KEYCODE_C,			// C
	KEYCODE_V,			// V
	0,
	KEYCODE_B,			// B
	KEYCODE_Q,			// Q
	KEYCODE_W,			// W
	KEYCODE_E,			// E
	KEYCODE_R,			// R
	/* 0x10 - 0x1f */
	KEYCODE_Y,			// Y
	KEYCODE_T,			// T
	KEYCODE_1,			// 1 !
	KEYCODE_2,			// 2 "
	KEYCODE_3,			// 3 #
	KEYCODE_4,			// 4 $
	KEYCODE_6,			// 6 &
	KEYCODE_5,			// 5 %
	KEYCODE_CARET,		// ~ ^ (JP) + = (US)
	KEYCODE_9,			// 9 )
	KEYCODE_7,			// 7 '
	KEYCODE_MINUS,		// - = (JP) - _ (US)
	KEYCODE_8,			// 8 (
	KEYCODE_0,			// 0
	KEYCODE_LBRACKET,	// [ { (JP) ] } (US)
	KEYCODE_O,			// O
	/* 0x20 - 0x2f */
	KEYCODE_U,			// U
	KEYCODE_AT,			// @ ` (JP) [ { (US)
	KEYCODE_I,			// I
	KEYCODE_P,			// P
	KEYCODE_RETURN,		// Enter
	KEYCODE_L,			// L
	KEYCODE_J,			// J
	KEYCODE_COLON,		// : * (JP) " ' (US)
	KEYCODE_K,			// K
	KEYCODE_SEMICOLON,	// ; + (JP) : ; (US)
	KEYCODE_RBRACKET,	// ] } (JP) | \ (US)
	KEYCODE_COMMA,		// , <
	KEYCODE_SLASH,		// / ?
	KEYCODE_N,			// N
	KEYCODE_M,			// M
	KEYCODE_PERIOD,		// . >
	/* 0x30 - 0x3f */
	KEYCODE_TAB,		// Tab
	KEYCODE_SPACE,		// Space
	KEYCODE_GRAVE,		// Kanji `
	KEYCODE_BACKSPACE,	// BackSpace
	0, //KEYCODE_IB_ENTER,		// iBook Enter
	KEYCODE_ESCAPE,		// ESC
	KEYCODE_RGUI,		// Right command
	KEYCODE_LGUI,		// Left command
	KEYCODE_LSHIFT,		// Left Shift
	KEYCODE_CAPSLOCK,	// CapsLock
	KEYCODE_LALT,		// Left Alt
	KEYCODE_LCTRL,		// Left Ctrl
	KEYCODE_RSHIFT,		// Right Shift
	KEYCODE_RALT,		// Right Alt
	KEYCODE_RCTRL,		// Right Ctrl
	KEYCODE_FUNCTION,	// fn
	/* 0x40 - 0x4f */
	KEYCODE_F17,		// F17
	KEYCODE_KP_PERIOD,	// num .
	0,
	KEYCODE_KP_MULTIPLY,// num *
	0,
	KEYCODE_KP_PLUS,	// num +
	0,
	KEYCODE_NUMLOCK,	// clear
	0,
	0,
	0,
	KEYCODE_KP_DIVIDE,	// num /
	KEYCODE_KP_ENTER,	// num enter
	KEYCODE_KP_MINUS,	// num -
	0,
	KEYCODE_F18,		// F18
	/* 0x50 - 0x5f */
	KEYCODE_F19,		// F19
	KEYCODE_KP_EQUALS,	// num equal
	KEYCODE_KP_0,		// num 0
	KEYCODE_KP_1,		// num 1
	KEYCODE_KP_2,		// num 2
	KEYCODE_KP_3,		// num 3
	KEYCODE_KP_4,		// num 4
	KEYCODE_KP_5,		// num 5
	KEYCODE_KP_6,		// num 6
	KEYCODE_KP_7,		// num 7
	0,
	KEYCODE_KP_8,		// num 8
	KEYCODE_KP_9,		// num 9
	KEYCODE_BACKSLASH,	// yen(backslash)
	KEYCODE_UNDERSCORE,	// _
	0,
	/* 0x60 - 0x6f */
	KEYCODE_F5,			// F5
	KEYCODE_F6,			// F6
	KEYCODE_F7,			// F7
	KEYCODE_F3,			// F3
	KEYCODE_F8,			// F8
	KEYCODE_F9,			// F9
	KEYCODE_EISU,		// eisu
	KEYCODE_F11,		// F11
	KEYCODE_KANA,		// kana
	KEYCODE_F13,		// F13
	KEYCODE_F16,		// F16
	KEYCODE_F14,		// F14
	0,
	KEYCODE_F10,		// F10
	KEYCODE_MENU,		// menu
	KEYCODE_F12,		// F12
	/* 0x70 - 0x7f */
	0,
	KEYCODE_F15,		// F15
	KEYCODE_INSERT,		// insert
	KEYCODE_HOME,		// home
	KEYCODE_PAGEUP,		// page up
	KEYCODE_DELETE,		// delete x
	KEYCODE_F4,			// F4
	KEYCODE_END,		// end
	KEYCODE_F2,			// F2
	KEYCODE_PAGEDOWN,	// page down
	KEYCODE_F1,			// F1
	KEYCODE_LEFT,		// left
	KEYCODE_RIGHT,		// right
	KEYCODE_DOWN,		// down
	KEYCODE_UP,			// up
	KEYCODE_POWER		// power

//	KEYCODE_PAUSE,		// Pause / Break
//	KEYCODE_SCROLLLOCK,	// Scroll Lock
//	KEYCODE_KATAHIRA,	// Katakana / Hiragana (JP)
//	KEYCODE_UNDERSCORE,	// _
};

typedef struct st_ks2kc {
	short ks;
	short kc;
} ks2kc_t;

const ks2kc_t keysym2keycode[] = {
	{ 97,  KEYCODE_A },
	{ 301, KEYCODE_CAPSLOCK },
	{ 303, KEYCODE_RSHIFT },
	{ 304, KEYCODE_LSHIFT },
	{ 305, KEYCODE_RCTRL },
	{ 306, KEYCODE_LCTRL },
	{ 307, KEYCODE_RALT },
	{ 308, KEYCODE_LALT },
	{ 309, KEYCODE_RGUI },
	{ 310, KEYCODE_LGUI },
	{ -1, -1 }
};

#elif defined(__WXGTK__)

/// mapping from gdk keysym to original keycode
const uint8_t gdkkey2keycode[256] = {
	/* 0x00 - 0x0f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x10 - 0x1f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x20 - 0x2f */
	KEYCODE_SPACE,
	KEYCODE_1,
	KEYCODE_2,
	KEYCODE_3,
	KEYCODE_4,
	KEYCODE_5,
	KEYCODE_6,
	KEYCODE_7,
	KEYCODE_8,
	KEYCODE_9,
	KEYCODE_COLON, //KEYCODE_AT,
	KEYCODE_SEMICOLON, //KEYCODE_PLUS,
	KEYCODE_COMMA,
	KEYCODE_MINUS,
	KEYCODE_PERIOD,
	KEYCODE_SLASH,
	/* 0x30 - 0x3f */
	KEYCODE_0,
	KEYCODE_1,
	KEYCODE_2,
	KEYCODE_3,
	KEYCODE_4,
	KEYCODE_5,
	KEYCODE_6,
	KEYCODE_7,
	KEYCODE_8,
	KEYCODE_9,
	KEYCODE_COLON,
	KEYCODE_SEMICOLON,
	KEYCODE_COMMA, //KEYCODE_LESS
	KEYCODE_MINUS, //KEYCODE_EQUAL
	KEYCODE_PERIOD, //KEYCODE_GREATER
	KEYCODE_SLASH, //KEYCODE_QUESTION
	/* 0x40 - 0x4f */
	KEYCODE_AT,
	KEYCODE_A,
	KEYCODE_B,
	KEYCODE_C,
	KEYCODE_D,
	KEYCODE_E,
	KEYCODE_F,
	KEYCODE_G,
	KEYCODE_H,
	KEYCODE_I,
	KEYCODE_J,
	KEYCODE_K,
	KEYCODE_L,
	KEYCODE_M,
	KEYCODE_N,
	KEYCODE_O,
	/* 0x50 - 0x5f */
	KEYCODE_P,
	KEYCODE_Q,
	KEYCODE_R,
	KEYCODE_S,
	KEYCODE_T,
	KEYCODE_U,
	KEYCODE_V,
	KEYCODE_W,
	KEYCODE_X,
	KEYCODE_Y,
	KEYCODE_Z,
	KEYCODE_LBRACKET,
	KEYCODE_BACKSLASH,
	KEYCODE_RBRACKET,
	KEYCODE_CARET,
	KEYCODE_UNDERSCORE,
	/* 0x60 - 0x6f */
	KEYCODE_AT,
	KEYCODE_A,
	KEYCODE_B,
	KEYCODE_C,
	KEYCODE_D,
	KEYCODE_E,
	KEYCODE_F,
	KEYCODE_G,
	KEYCODE_H,
	KEYCODE_I,
	KEYCODE_J,
	KEYCODE_K,
	KEYCODE_L,
	KEYCODE_M,
	KEYCODE_N,
	KEYCODE_O,
	/* 0x70 - 0x7f */
	KEYCODE_P,
	KEYCODE_Q,
	KEYCODE_R,
	KEYCODE_S,
	KEYCODE_T,
	KEYCODE_U,
	KEYCODE_V,
	KEYCODE_W,
	KEYCODE_X,
	KEYCODE_Y,
	KEYCODE_Z,
	KEYCODE_LBRACKET,
	KEYCODE_BACKSLASH,
	KEYCODE_RBRACKET,
	KEYCODE_CARET,
	KEYCODE_UNDERSCORE,
	/* 0x80 - 0x8f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x90 - 0x9f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xa0 - 0xaf */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xb0 - 0xbf */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xc0 - 0xcf */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xd0 - 0xdf */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xe0 - 0xef */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0xf0 - 0xff */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0
};

const uint8_t gdkkey2keycodeFF[256] = {
	/* 0x00 - 0x0f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	KEYCODE_BACKSPACE,
	KEYCODE_TAB,
	0,
	KEYCODE_CLEAR,
	0,
	KEYCODE_RETURN,
	0,
	0,
	/* 0x10 - 0x1f */
	0,
	0,
	0,
	KEYCODE_PAUSE,
	KEYCODE_SCROLLLOCK,
	KEYCODE_SYSREQ,
	0,
	0,
	0,
	KEYCODE_GRAVE,
	0,
	KEYCODE_ESCAPE,
	0,
	0,
	0,
	0,
	/* 0x20 - 0x2f */
	0,
	KEYCODE_GRAVE,
	KEYCODE_MUHENKAN,
	KEYCODE_HENKAN,
	0, //KEYCODE_ROMAJI,
	KEYCODE_KATAHIRA,
	KEYCODE_KATAHIRA,
	KEYCODE_KATAHIRA,
	KEYCODE_GRAVE,
	KEYCODE_GRAVE,
	KEYCODE_GRAVE,
	0,
	0,
	0,
	0,
	0,
	/* 0x30 - 0x3f */
	KEYCODE_CAPSLOCK,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x40 - 0x4f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x50 - 0x5f */
	KEYCODE_HOME,
	KEYCODE_LEFT,
	KEYCODE_UP,
	KEYCODE_RIGHT,
	KEYCODE_DOWN,
	KEYCODE_PAGEUP,
	KEYCODE_PAGEDOWN,
	KEYCODE_END,
	0, //KEYCODE_BGEIN,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	/* 0x60 - 0x6f */
	KEYCODE_SELECT,
	KEYCODE_PRINT,
	0, //KEYCODE_EXECUTE,
	KEYCODE_INSERT,
	KEYCODE_UNDO,
	0, //KEYCODE_REDO,
	KEYCODE_MENU,
	0, //KEYCODE_FIND,
	0, //KEYCODE_CANCEL,
	KEYCODE_HELP,
	0, //KEYCODE_BREAK,
	0,
	0,
	0,
	0,
	0,
	/* 0x70 - 0x7f */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	KEYCODE_NUMLOCK,
	/* 0x80 - 0x8f */
	0, //KEYCODE_KP_SPACE,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0, //KEYCODE_KP_TAB,
	0,
	0,
	0,
	KEYCODE_KP_ENTER,
	0,
	0,
	/* 0x90 - 0x9f */
	0,
	0, //KEYCODE_KP_F1,
	0, //KEYCODE_KP_F2,
	0, //KEYCODE_KP_F3,
	0, //KEYCODE_KP_F4,
	KEYCODE_KP_7, //KEYCODE_KP_HOME,
	KEYCODE_KP_4, //KEYCODE_KP_LEFT,
	KEYCODE_KP_8, //KEYCODE_KP_UP,
	KEYCODE_KP_6, //KEYCODE_KP_RIGHT,
	KEYCODE_KP_2, //KEYCODE_KP_DOWN,
	KEYCODE_KP_9, //KEYCODE_KP_PAGEUP,
	KEYCODE_KP_3, //KEYCODE_KP_PAGEDOWN,
	KEYCODE_KP_1, //KEYCODE_KP_END,
	0, //KEYCODE_KP_BEGIN,
	KEYCODE_KP_0, //KEYCODE_KP_INSERT,
	KEYCODE_KP_PERIOD, //KEYCODE_KP_DELETE,
	/* 0xa0 - 0xaf */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	KEYCODE_KP_MULTIPLY,
	KEYCODE_KP_PLUS,
	KEYCODE_KP_DIVIDE,
	KEYCODE_KP_MINUS,
	KEYCODE_KP_PERIOD,
	KEYCODE_KP_DIVIDE,
	/* 0xb0 - 0xbf */
	KEYCODE_KP_0,
	KEYCODE_KP_1,
	KEYCODE_KP_2,
	KEYCODE_KP_3,
	KEYCODE_KP_4,
	KEYCODE_KP_5,
	KEYCODE_KP_6,
	KEYCODE_KP_7,
	KEYCODE_KP_8,
	KEYCODE_KP_9,
	0,
	0,
	0,
	0,
	KEYCODE_F1,
	KEYCODE_F2,
	/* 0xc0 - 0xcf */
	KEYCODE_F3,
	KEYCODE_F4,
	KEYCODE_F5,
	KEYCODE_F6,
	KEYCODE_F7,
	KEYCODE_F8,
	KEYCODE_F9,
	KEYCODE_F10,
	KEYCODE_F11,
	KEYCODE_F12,
	KEYCODE_F13,
	KEYCODE_F14,
	KEYCODE_F15,
	KEYCODE_F16,
	KEYCODE_F17,
	KEYCODE_F18,
	/* 0xd0 - 0xdf */
	KEYCODE_F19,
	KEYCODE_F20,
	KEYCODE_F21,
	KEYCODE_F22,
	0, //KEYCODE_F23,
	0, //KEYCODE_F24,
	0, //KEYCODE_F25,
	0, //KEYCODE_F26,
	0, //KEYCODE_F27,
	0, //KEYCODE_F28,
	0, //KEYCODE_F29,
	0, //KEYCODE_F30,
	0,
	0,
	0,
	0,
	/* 0xe0 - 0xef */
	0,
	KEYCODE_LSHIFT,
	KEYCODE_RSHIFT,
	KEYCODE_LCTRL,
	KEYCODE_RCTRL,
	KEYCODE_CAPSLOCK,
	0, //KEYCODE_SHIFTLOCK,
	KEYCODE_LGUI,
	KEYCODE_RGUI,
	KEYCODE_LALT,
	KEYCODE_RALT,
	KEYCODE_LSUPER,
	KEYCODE_RSUPER,
	0,
	0,
	0,
	/* 0xf0 - 0xff */
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0
};

#endif

#ifdef USE_AUTO_KEY

#ifdef USE_EMU_INHERENT_SPEC
#define USE_AUTO_KEY_VM_CODE
#ifdef USE_AUTO_KEY_VM_CODE
#define AUTO_KEY_SHIFT	0x87
#define AUTO_KEY_RETURN	0xaf
#define AUTO_KEY_KANA	0x8a
#define AUTO_KEY_GRAPH	0x8b
#define AUTO_KEY_NONE	0xff
#define AUTO_KEY_MASK		0x00ff
#define AUTO_KEY_SHIFT_MASK	0x0100
#define AUTO_KEY_GRAPH_MASK	0x0200
#define AUTO_KEY_UPPER_MASK	0x0400
#define AUTO_KEY_LOWER_MASK	0x0800
#else
#define AUTO_KEY_SHIFT	WXK_SHIFT
#define AUTO_KEY_RETURN	0x0d
#define AUTO_KEY_KANA	WXK_TAB
#define AUTO_KEY_GRAPH	WXK_F8
#define AUTO_KEY_NONE	0x00
#define AUTO_KEY_MASK		0x03ff
#define AUTO_KEY_SHIFT_MASK	0x0400
#define AUTO_KEY_GRAPH_MASK	0x0800
#define AUTO_KEY_UPPER_MASK	0		// unused
#define AUTO_KEY_LOWER_MASK	0		// unused
#endif

#else

#define AUTO_KEY_SHIFT	WXK_SHIFT
#define AUTO_KEY_RETURN	0x0d
#define AUTO_KEY_KANA	WXK_TAB
#define AUTO_KEY_NONE	0x00
#define AUTO_KEY_MASK		0x03ff
#define AUTO_KEY_SHIFT_MASK	0x0100
#define AUTO_KEY_KANA_MASK	0x0200
#define AUTO_KEY_UPPER_MASK	0x0400
#define AUTO_KEY_LOWER_MASK	0x0800
#endif

static const int autokey_table[256] = {
#ifdef USE_EMU_INHERENT_SPEC
	// 0x100: shift
	// 0x200: graph
	// 0x400: alphabet
	// 0x800: ALPHABET
	// 0x1000: katakana
	// 0x2000: hiragana
#ifdef USE_AUTO_KEY_VM_CODE	// use vm key scan code 0x80 - 0xfe
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x0af,0x000,0x000,0x0af,0x000,0x000,
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x08c,0x083,0x085,0x081,0x084,
	0x080,0x19a,0x19b,0x197,0x191,0x199,0x192,0x190,0x193,0x19c,0x1b6,0x1b4,0x0c3,0x096,0x0cc,0x0c4,
	0x094,0x09a,0x09b,0x097,0x091,0x099,0x092,0x090,0x093,0x09c,0x0b6,0x0b4,0x1c3,0x196,0x1cc,0x1c4,
	0x0a6,0x4b8,0x4c9,0x4cb,0x4bb,0x4ab,0x4b1,0x4b9,0x4b2,0x4a3,0x4b0,0x4b3,0x4bc,0x4c0,0x4c2,0x4ac,
	0x4a4,0x4a8,0x4a1,0x4ba,0x4a9,0x4a0,0x4c1,0x4aa,0x4ca,0x4a2,0x4c8,0x0a5,0x09f,0x0b5,0x095,0x1c6,
	0x000,0x8b8,0x8c9,0x8cb,0x8bb,0x8ab,0x8b1,0x8b9,0x8b2,0x8a3,0x8b0,0x8b3,0x8bc,0x8c0,0x8c2,0x8ac,
	0x8a4,0x8a8,0x8a1,0x8ba,0x8a9,0x8a0,0x8c1,0x8aa,0x8ca,0x8a2,0x8c8,0x000,0x000,0x000,0x000,0x000,
	// hiragana1
	0x2c8,0x2ca,0x2cb,0x2c1,0x2c9,0x2c2,0x2c0,0x2c3,0x2cc,0x2bc,0x2ac,0x29c,0x293,0x2a3,0x2b3,0x2bd,
	0x2cd,0x28d,0x2be,0x2b7,0x2b4,0x2bf,0x2cf,0x2c4,0x29d,0x28e,0x2c7,0x2ce,0x2a1,0x2a9,0x2b1,0x2b9,
	// katakana
	0x029a,0x11cc,0x11a5,0x11b5,0x11c3,0x11c4,0x1194,0x1197,0x11ab,0x1191,0x1199,0x1192,0x1190,0x1193,0x119c,0x11c8,
	0x109f,0x1097,0x10ab,0x1091,0x1099,0x1092,0x10a9,0x10b9,0x10b2,0x10b6,0x10c9,0x10ca,0x10bb,0x10a1,0x10a4,0x10cb,
	0x10a8,0x10b8,0x10c8,0x10aa,0x10ba,0x10a0,0x10a3,0x109a,0x10c3,0x10b3,0x10b1,0x10c1,0x109b,0x1095,0x1096,0x10b0,
	0x10c2,0x10b5,0x10c4,0x10c0,0x1090,0x1093,0x109c,0x10ac,0x10bc,0x10cc,0x10b4,0x10c6,0x1094,0x10a2,0x10a6,0x10a5,
	// hiragana2
	0x296,0x2a6,0x2b6,0x2c6,0x2a2,0x2a0,0x2b2,0x2b0,0x2a8,0x2b8,0x2aa,0x2ba,0x2ab,0x2bb,0x2c5,0x28f,
	0x2a7,0x29f,0x29b,0x297,0x291,0x299,0x292,0x290,0x2ad,0x294,0x2a4,0x295,0x2a5,0x20a2,0x2b5,0x000,
#else
	// 0x400: shift
	// 0x800: graph
	// 0x1000: katakana
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x00d,0x000,0x000,0x00d,0x000,0x000,
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,
	0x020,0x431,0x432,0x433,0x434,0x435,0x436,0x437,0x438,0x439,0x43a,0x43b,0x02c,0x02d,0x02e,0x02f,
	0x030,0x031,0x032,0x033,0x034,0x035,0x036,0x037,0x038,0x039,0x03a,0x03b,0x42c,0x42d,0x42e,0x42f,
	0x040,0x041,0x042,0x043,0x044,0x045,0x046,0x047,0x048,0x049,0x04a,0x04b,0x04c,0x04d,0x04e,0x04f,
	0x050,0x051,0x052,0x053,0x054,0x055,0x056,0x057,0x058,0x059,0x05a,0x05b,0x05c,0x05d,0x05e,0x05f,
	0x000,0x441,0x442,0x443,0x444,0x445,0x446,0x447,0x448,0x449,0x44a,0x44b,0x44c,0x44d,0x44e,0x44f,
	0x470,0x451,0x452,0x453,0x454,0x455,0x456,0x457,0x458,0x459,0x45a,0x000,0x000,0x000,0x000,0x000,
	// hiragana1
	0x85a,0x858,0x843,0x856,0x842,0x84e,0x84d,0x82c,0x82e,0x84c,0x84f,0x839,0x838,0x849,0x84b,0x949,
	0x946,0x94c,0x94a,0x948,0x83b,0x986,0x987,0x82f,0x94b,0x94d,0x945,0x947,0x852,0x854,0x846,0x847,
	// katakana
	0x0831,0x142e,0x145b,0x145d,0x142c,0x142f,0x1430,0x1433,0x1465,0x1434,0x1435,0x1436,0x1437,0x1438,0x1439,0x147a,
	0x105c,0x1033,0x1045,0x1034,0x1035,0x1036,0x1054,0x1047,0x1048,0x103a,0x1042,0x1058,0x1044,0x1052,0x1050,0x1043,
	0x1051,0x1041,0x105a,0x1057,0x1053,0x1055,0x1049,0x1031,0x102c,0x104b,0x1046,0x1056,0x1032,0x105e,0x102d,0x104a,
	0x104e,0x105d,0x102f,0x104d,0x1037,0x1038,0x1039,0x104f,0x104c,0x102e,0x103b,0x105f,0x1030,0x1059,0x1040,0x105b,
	// hiragana2
	0x82d,0x840,0x83a,0x85f,0x859,0x855,0x848,0x84a,0x851,0x841,0x857,0x853,0x845,0x844,0x988,0x983,
	0x944,0x85c,0x832,0x833,0x834,0x835,0x836,0x837,0x987,0x830,0x850,0x85e,0x85b,0x2059,0x85d,0x000,
#endif
#else
	// 0x100: shift
	// 0x200: kana
	// 0x400: alphabet
	// 0x800: ALPHABET
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x00d,0x000,0x000,0x00d,0x000,0x000,
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,
	0x020,0x131,0x132,0x133,0x134,0x135,0x136,0x137,0x138,0x139,0x1ba,0x1bb,0x0bc,0x0bd,0x0be,0x0bf,
	0x030,0x031,0x032,0x033,0x034,0x035,0x036,0x037,0x038,0x039,0x0ba,0x0bb,0x1bc,0x1bd,0x1be,0x1bf,
	0x0c0,0x441,0x442,0x443,0x444,0x445,0x446,0x447,0x448,0x449,0x44a,0x44b,0x44c,0x44d,0x44e,0x44f,
	0x450,0x451,0x452,0x453,0x454,0x455,0x456,0x457,0x458,0x459,0x45a,0x0db,0x0dc,0x0dd,0x0de,0x1e2,
	0x1c0,0x841,0x842,0x843,0x844,0x845,0x846,0x847,0x848,0x849,0x84a,0x84b,0x84c,0x84d,0x84e,0x84f,
	0x850,0x851,0x852,0x853,0x854,0x855,0x856,0x857,0x858,0x859,0x85a,0x1db,0x1dc,0x1dd,0x1de,0x000,
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,
	// kana -->
	0x000,0x3be,0x3db,0x3dd,0x3bc,0x3bf,0x330,0x333,0x345,0x334,0x335,0x336,0x337,0x338,0x339,0x35a,
	0x2dc,0x233,0x245,0x234,0x235,0x236,0x254,0x247,0x248,0x2ba,0x242,0x258,0x244,0x252,0x250,0x243,
	0x251,0x241,0x25a,0x257,0x253,0x255,0x249,0x231,0x2bc,0x24b,0x246,0x256,0x232,0x2de,0x2bd,0x24a,
	0x24e,0x2dd,0x2bf,0x24d,0x237,0x238,0x239,0x24f,0x24c,0x2be,0x2bb,0x2e2,0x230,0x259,0x2c0,0x2db,
	// <--- kana
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,
	0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000
#endif
};
#endif

#endif /* WXW_INPUT_H */
