/** @file wxw_socket.cpp

	Skelton for retropc emulator
	SDL edition

	@author Sasaji
	@date   2013.12.06

	@brief [ wxw socket ]
*/

#include <wx/wx.h>
#include "wxw_emu.h"
#include "../../vm/vm.h"
#include "../../vm/device.h"
#include "wxw_socket.h"
#include "wxw_main.h"

static MyConnection *conn;

void EMU_OSD::EMU_SOCKET()
{
	conn = new MyConnection(this, SOCKET_MAX);

	// init sockets
	for(int i = 0; i < SOCKET_MAX; i++) {
		socs[i].device = NULL;
		socs[i].is_tcp = false;
		socs[i].is_server = false;
		socs[i].socket_delay = 0;
		socs[i].recv_buffer = NULL;
		socs[i].recv_r_ptr = 0;
		socs[i].recv_w_ptr = 0;
	}
	socket_nums = 0;
}

void EMU_OSD::initialize_socket()
{
}

void EMU_OSD::release_socket()
{
	// release sockets
	for(int i = 0; i < SOCKET_MAX; i++) {
		delete[] socs[i].recv_buffer;
	}

	delete conn;
}

void EMU_OSD::socket_connected(int ch)
{
	// network is connected
	logging->out_debugf(_T("connected. ch=%d"), ch);
	if (socs[ch].device) socs[ch].device->network_connected(ch);
//	else vm->network_connected(ch);
}

void EMU_OSD::socket_disconnected(int ch)
{
	// network is disconnected
//	if(!socs[ch].socket_delay) {
//		socs[ch].socket_delay = 1;//56;
//	}
	logging->out_debugf(_T("disconnected. ch=%d"), ch);
}

void EMU_OSD::socket_writeable(int ch)
{
	// network is writeable
	logging->out_debugf(_T("writeable. ch=%d"), ch);
	if (socs[ch].device) socs[ch].device->network_writeable(ch);
//	else vm->network_writeable(ch);
}

void EMU_OSD::socket_readable(int ch)
{
	// network is readable
	logging->out_debugf(_T("readable. ch=%d"), ch);
	if (socs[ch].device) socs[ch].device->network_readable(ch);
//	else vm->network_readable(ch);
}

static int memcpy_remove_dup_iac(char *dst, const char *src, int size)
{
	int pos=0;
	for(int i=0; i<size;) {
		if (src[i] == (char)0xff && i+1 == size) {
			break;
		}
		dst[pos++]=src[i];
		if (src[i] == (char)0xff && src[i+1] == (char)0xff) {
			i++;
		}
		i++;
	}
	return pos;
}

void EMU_OSD::update_socket()
{
	// buffer copy
	for(int i = 0; i < SOCKET_MAX; i++) {
		struct st_soc *soc = &socs[i];
		if(soc->recv_r_ptr < soc->recv_w_ptr) {
			if (soc->device) {
				// get buffer
				int size0 = 0;
				int size1 = 0;
				int flags = 0;
				uint8_t* buf0 = soc->device->get_recvbuffer0(i, &size0, &size1, &flags);
				//	buf0 = vm->get_recvbuffer0(i, &size0, &size1, &flags);
				uint8_t* buf1 = soc->device->get_recvbuffer1(i);
				//	buf1 = vm->get_recvbuffer1(i);

				int size = soc->recv_w_ptr - soc->recv_r_ptr;
				if(size > size0 + size1) {
					size = size0 + size1;
				}
				char* src = &soc->recv_buffer[soc->recv_r_ptr];
				soc->recv_r_ptr += size;

				if(size <= size0) {
					if (!(flags & 1)) {
						memcpy(buf0, src, size);
					} else {
						// escape telnet iac
						size = memcpy_remove_dup_iac((char *)buf0, src, size);
					}
				}
				else {
					if (!(flags & 1)) {
						memcpy(buf0, src, size0);
						memcpy(buf1, src + size0, size - size0);
					} else {
						// escape telnet iac
						size0 = memcpy_remove_dup_iac((char *)buf0, src, size0);
						size = memcpy_remove_dup_iac((char *)buf1, src + size0, size - size0);
					}
				}
				soc->device->inc_recvbuffer_ptr(i, size);
				//	vm->inc_recvbuffer_ptr(i, size);
			}
		}
		else if(soc->socket_delay != 0) {
			if(--soc->socket_delay == 0) {
				disconnect_socket(i);
//				if (soc->device) soc->device->network_disconnected(i);
//				else vm->network_disconnected(i);
			}
		}
	}
}

void EMU_OSD::socket_accepted(int ch, int new_ch, bool tcp)
{
	if (ch < 0 || SOCKET_MAX <= ch) return;


	socs[new_ch].is_tcp=tcp;
	socs[new_ch].device = socs[ch].device;
	logging->out_debugf(_T("accepted. ch=%d new_ch=%d"), ch, new_ch);
	if (socs[ch].device) socs[ch].device->network_accepted(ch, new_ch);
//	else vm->network_accepted(ch, new_ch);
}

/// @note called by main thread
bool EMU_OSD::init_socket_tcp(int ch, DEVICE *dev, bool server)
{
	if (ch < 0 || SOCKET_MAX <= ch) {
		logging->out_logf(LOG_ERROR, _T("EMU_OSD::init_socket_tcp: invalid socket channel. ch=%d"), ch);
		return false;
	}

	struct st_soc *soc = &socs[ch];
	disconnect_socket(ch);

	// reserve channel for tcp connection
	soc->is_tcp = true;

	soc->recv_r_ptr = soc->recv_w_ptr = 0;
	soc->device = dev;
	return true;
}

/// @note called by main thread
bool EMU_OSD::init_socket_udp(int ch, DEVICE *dev, bool server)
{
	if (ch < 0 || SOCKET_MAX <= ch) {
		logging->out_logf(LOG_ERROR, _T("EMU_OSD::init_socket_udp: invalid socket channel. ch=%d"), ch);
		return false;
	}

	struct st_soc *soc = &socs[ch];

	disconnect_socket(ch);

	// reserve channel for udp connection
	soc->is_tcp = false;

	soc->recv_r_ptr = soc->recv_w_ptr = 0;
	soc->device = dev;
	return true;
}

bool EMU_OSD::connect_socket(int ch, const _TCHAR *hostname, int port, bool server)
{
	return conn->Connect(ch, hostname, port, server, socs[ch].is_tcp);
}

void EMU_OSD::disconnect_socket(int ch)
{
	if (ch < 0 || SOCKET_MAX <= ch) return;

	conn->Disconnect(ch);
}

int EMU_OSD::get_socket_channel()
{
	return conn->GetSocketChannel();
}

bool EMU_OSD::listen_socket(int ch)
{
	return false;
}

/// @note called by emu thread
void EMU_OSD::send_data_tcp(int ch)
{
	if (ch < 0 || SOCKET_MAX <= ch) return;

	if(socs[ch].is_tcp) {
		send_data_(ch);
	}
}

/// @note called by emu thread
void EMU_OSD::send_data_udp(int ch, const _TCHAR *hostname, int port)
{
	if(!socs[ch].is_tcp) {
		send_data_(ch, hostname, port);
	}
}

/// @note called by emu thread
void EMU_OSD::send_data(int ch)
{
	send_data_(ch);
}

/// @note usually called by emu thread
/// @note called by main thread at first when connection succeeded.
void EMU_OSD::send_data_(int ch, const _TCHAR *hostname, int port)
{
	if (ch < 0 || SOCKET_MAX <= ch) return;

	struct st_soc *soc = &socs[ch];

	// get send buffer and data size
	int size;
	int flags = 0;
	uint8_t* buf;
	uint8_t bufcnvd[1024];

	// loop while send buffer is not empty
	while(soc->device) {
		buf = soc->device->get_sendbuffer(ch, &size, &flags);
		//	buf = vm->get_sendbuffer(ch, &size);

		if(!buf || !size) {
			break;
		}
		if (flags & 1) {
			// escape telnet iac
			int pos=0;
			for(int i=0; i<size && pos<1023; i++) {
				bufcnvd[pos++]=buf[i];
				if (buf[i] == 0xff) {
					bufcnvd[pos++]=buf[i];
				}
			}
			size=pos;
			buf=bufcnvd;
		}
		if(soc->is_tcp) {
			int sent_size = (int)conn->Send(ch, (const char *)buf, size);
			if(sent_size < size) {
				// error
				disconnect_socket(ch);
				socket_disconnected(ch);
				break;
			}
			size = sent_size;
		} else {
			int sent_size = (int)conn->SendTo(ch, hostname, port, (const void *)buf, size);
			if(sent_size < size) {
				// error
				disconnect_socket(ch);
				socket_disconnected(ch);
				break;
			}
			size = sent_size;
		}
		soc->device->inc_sendbuffer_ptr(ch, size);
		//	vm->inc_sendbuffer_ptr(ch, size);
	}
}

/// Receive data from socket, and store to first buffer to be able to read from emu devices.
/// @note called by main thread
void EMU_OSD::recv_data(int ch)
{
	if (ch < 0 || SOCKET_MAX <= ch) return;
	struct st_soc *soc = &socs[ch];

	if (soc->recv_buffer == NULL) {
		soc->recv_buffer = new char[SOCKET_BUFFER_MAX];
	}

	if(soc->is_tcp) {
		int size = SOCKET_BUFFER_MAX - soc->recv_w_ptr;
		char* buf = &soc->recv_buffer[soc->recv_w_ptr];
		int recv_size = (int)conn->Recv(ch, buf, size);
		if(recv_size <= 0) {
			disconnect_socket(ch);
			socket_disconnected(ch);
			return;
		}
		soc->recv_w_ptr += recv_size;
	}
	else {
		int size = SOCKET_BUFFER_MAX - soc->recv_w_ptr;
		char* buf = &soc->recv_buffer[soc->recv_w_ptr];
		if(size < 8) {
			return;
		}
		char ipaddr[128];
		int port;
		int recv_size = (int)conn->RecvFrom(ch, ipaddr, &port, (void *)(buf+8), size);
		if(recv_size <= 0) {
			disconnect_socket(ch);
			socket_disconnected(ch);
			return;
		}
//		size += 8;
		buf[0] = (char)(recv_size >> 8);
		buf[1] = (char)recv_size;
		int val[4];
		int num = sscanf(ipaddr, "%d.%d.%d.%d", &val[0], &val[1], &val[2], &val[3]);
		if (num == 4) {
			buf[2] = (char)val[0];
			buf[3] = (char)val[1];
			buf[4] = (char)val[2];
			buf[5] = (char)val[3];
		}
		buf[6] = (char)port;
		buf[7] = (char)(port >> 8);
		soc->recv_w_ptr += recv_size;
	}
}

const void *EMU_OSD::get_socket(int ch) const
{
	return NULL;
}




typedef wxIPV4address MyIPaddress;

/*******************************************************
 * Network
 */
MyConnection::MyConnection(EMU *parent, int max_sockets)
	: wxObject()
{
	emu = parent;
	mSocketMax = max_sockets;

	mSocket = new MySocket *[mSocketMax];
	mSocketUsed = new bool[mSocketMax];
	mSocketNum = 0;
	for(int i=0; i<mSocketMax; i++) {
		mSocket[i] = NULL;
		mSocketUsed[i] = false;
	}
}
MyConnection::~MyConnection()
{
	for(int i=0; i<mSocketMax; i++) {
		delete mSocket[i];
	}
	delete [] mSocketUsed;
	delete [] mSocket;
}
bool MyConnection::Connect(const _TCHAR *hostname, int port, bool server, bool tcp)
{
	int new_ch = GetSocketChannel();
	if (new_ch < 0) {
		return false;
	}
	return Connect(new_ch, hostname, port, server);
}
bool MyConnection::Connect(int new_ch, const _TCHAR *hostname, int port, bool server, bool tcp)
{
	if (new_ch < 0 || new_ch >= mSocketMax || mSocketUsed[new_ch]) {
		logging->out_logf(LOG_ERROR, _T("MyConnection: new_ch:%d Already used."), new_ch);
		return false;
	}
	mSocket[new_ch]=new MySocket(this, new_ch, hostname, port, server, tcp);
	if (!mSocket[new_ch]->IsOk()) {
		delete mSocket[new_ch];
		mSocket[new_ch]=NULL;
		return false;
	}
	mSocketUsed[new_ch]=true;
	return true;
}
bool MyConnection::AcceptSocket(int ch, wxSocketBase *sock, bool tcp)
{
	int new_ch = GetSocketChannel();
	if (new_ch < 0) {
		return false;
	}
	mSocket[new_ch]=new MySocket(this, new_ch, sock);
	if (!mSocket[new_ch]->IsOk()) {
		delete mSocket[new_ch];
		mSocket[new_ch]=NULL;
		return false;
	}
	mSocketUsed[new_ch]=true;
	emu->socket_accepted(ch, new_ch, tcp);
	return true;
}
void MyConnection::Disconnect(int ch)
{
	if (ch >= 0 && ch < mSocketMax && mSocketUsed[ch]) {
		mSocket[ch]->Disconnect();
	}
}
int MyConnection::GetSocketChannel()
{
	int new_ch = -1;
	for(int i = 0; i < mSocketMax; i++) {
		if (!mSocketUsed[i]) {
			new_ch = i;
			break;
		}
	}
	if (new_ch < 0) {
		logging->out_log(LOG_ERROR, _T("MyConnection: Socket is full."));
	}
	return new_ch;
}
size_t MyConnection::Send(int ch, const char *buffer, size_t len)
{
	if (ch >= 0 && ch < mSocketMax && mSocketUsed[ch]) {
		return mSocket[ch]->Send(buffer, len);
	} else {
		return 0;
	}
}
size_t MyConnection::Recv(int ch, char *buffer, size_t len)
{
	if (ch >= 0 && ch < mSocketMax && mSocketUsed[ch]) {
		return mSocket[ch]->Recv(buffer, len);
	} else {
		return 0;
	}
}
size_t MyConnection::SendTo(int ch, const _TCHAR *hostname, int port, const void *buffer, size_t len)
{
	if (ch >= 0 && ch < mSocketMax && mSocketUsed[ch]) {
		return mSocket[ch]->SendTo(hostname, port, buffer, len);
	} else {
		return 0;
	}
}
size_t MyConnection::RecvFrom(int ch, char *ipaddr, int *port, void *buffer, size_t len)
{
	if (ch >= 0 && ch < mSocketMax && mSocketUsed[ch]) {
		return mSocket[ch]->RecvFrom(ipaddr, port, buffer, len);
	} else {
		return 0;
	}
}
void MyConnection::OnDestroy(int ch)
{
	if (ch >= 0 && ch < mSocketMax && mSocketUsed[ch]) {
		delete mSocket[ch];
		mSocket[ch]=NULL;
		mSocketUsed[ch]=false;
	}
}

//
BEGIN_EVENT_TABLE(MySocket, wxEvtHandler)
	EVT_SOCKET(IDS_ACCEPT_ID, MySocket::OnAcceptEvent)
	EVT_SOCKET(IDS_CONNECT_ID, MySocket::OnConnectEvent)
	EVT_SOCKET(IDS_SOCKET_ID, MySocket::OnSocketEvent)
	EVT_TIMER(IDS_TIMER_ID, MySocket::OnTimerEvent)
END_EVENT_TABLE()

// create new socket
MySocket::MySocket(MyConnection *parent, int ch, const wxString &hostname, int port, bool server, bool tcp)
	: wxEvtHandler()
{
	m_ch = ch;
	m_ok = false;
	conn = parent;
	emu = parent->GetEMU();
//	m_busy = false;

	mSocket = NULL;
	m_server = server;
	m_tcp = tcp;

	if (server) {
		// create server socket

		MyIPaddress addr;
		addr.Hostname(hostname);
		addr.Service(port);

		// Create the socket
		mSocket = (wxSocketBase *)new wxSocketServer(addr);
		// We use IsOk() here to see if the server is really listening
		if (!mSocket->IsOk()) {
			logging->out_logf(LOG_ERROR, _T("MySocket%d: Couldn't listen at %ls:%d"), m_ch, hostname.wc_str(), port);
			return;
		}
		MyIPaddress addrReal;
		if (!mSocket->GetLocal(addrReal)) {
			logging->out_logf(LOG_ERROR, _T("MySocket%d: Couldn't get the address we bound to"), m_ch);
			return;
		}
		logging->out_logf(LOG_DEBUG, _T("MySocket%d: Server listening at %ls:%u"), m_ch, addrReal.IPAddress().wc_str(), addrReal.Service());

		// prepared
//		mTimer.SetOwner(this, IDS_TIMER_ID);
//		mTimerType = IDT_CONNECTED;
//		mTimer.StartOnce(30);

		// wait connection from client
		mSocket->SetEventHandler(*this, IDS_ACCEPT_ID);
		mSocket->SetNotify(wxSOCKET_CONNECTION_FLAG);
		mSocket->Notify(true);

	} else {
		// create client socket

		if (m_tcp) {
			// Create the socket
			mSocket = (wxSocketBase *)new wxSocketClient();
			// IsOK always failed
//			if (!mSocket->IsOk()) {
//				logging->out_logf(LOG_ERROR, _T("MySocket%d: Couldn't create socket client."), m_ch);
//				return;
//			}

			// connect to server
			if (!Connect(hostname, port)) {
				logging->out_logf(LOG_ERROR, _T("MySocket%d: Couldn't connect to server."), m_ch);
				return;
			}
		} else {

			MyIPaddress addr;
			addr.Hostname(hostname);
			addr.Service(port);

			// Create udp socket
			mSocket = (wxSocketBase *)new wxDatagramSocket(addr);
		}
	}
	m_ok = true;
	return;
}
// set socket from accepted sock
MySocket::MySocket(MyConnection *parent, int ch, wxSocketBase *sock)
	: wxEvtHandler()
{
	m_ch = ch;
	m_ok = false;
	conn = parent;
	emu = parent->GetEMU();
//	m_busy = false;

	mSocket = sock;
	m_server = false;

	// connected and writeable
	mTimer.SetOwner(this, IDS_TIMER_ID);
	mTimerType = IDT_CONNECTED;
	mTimer.StartOnce(30);

	// setup receive/lost event
	sock->SetEventHandler(*this, IDS_SOCKET_ID);
	sock->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG);
	sock->Notify(true);

	logging->out_logf(LOG_DEBUG, _T("MySocket%d: Accepted client."), m_ch);

	m_ok = true;
	return;
}
MySocket::~MySocket()
{
	// destory and delete mSocket
	mSocket->Destroy();
	mSocket = NULL;
	logging->out_logf(LOG_DEBUG, _T("MySocket%d: Destroy."), m_ch);
}

void MySocket::OnTimerEvent(wxTimerEvent& event)
{
	switch(mTimerType) {
	case IDT_CONNECTED:
		// client connected
		emu->socket_connected(m_ch);
		// and writeable
		emu->socket_writeable(m_ch);
		// send data
		emu->send_data(m_ch);
		break;
	}
}

void MySocket::OnAcceptEvent(wxSocketEvent& event)
{
	wxSocketBase *sock;

	sock = ((wxSocketServer *)mSocket)->Accept(false);
	if (sock) {
		MyIPaddress addr;
		if (!sock->GetPeer(addr)) {
			logging->out_logf(LOG_DEBUG, _T("MySocket%d: New connection from unknown client accepted."), m_ch);
		} else {
			logging->out_logf(LOG_DEBUG, _T("MySocket%d: New client connection from %ls:%u accepted"),
				   m_ch, addr.IPAddress().wc_str(), addr.Service());
		}

		if (!conn->AcceptSocket(m_ch, sock, m_tcp)) {
			sock->Destroy();
			logging->out_logf(LOG_ERROR, _T("MySocket%d: New client connection forcelly destroyed."), m_ch);
			return;
		}

	} else {
		logging->out_logf(LOG_ERROR, _T("MySocket%d: Accept Error"), m_ch);
		return;
	}
}

void MySocket::OnSocketEvent(wxSocketEvent& event)
{
	switch(event.GetSocketEvent()) {
	case wxSOCKET_INPUT:
		OnSocketInputEvent(event);
		break;
	case wxSOCKET_LOST:
		OnSocketLostEvent(event);
		break;
	default:
		break;
	}
}

void MySocket::OnSocketInputEvent(wxSocketEvent& event)
{
	// receive data
	emu->recv_data(m_ch);
	// readable
	emu->socket_readable(m_ch);

	// Enable input events again.
	mSocket->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);
}
void MySocket::OnSocketLostEvent(wxSocketEvent& event)
{
	logging->out_logf(LOG_DEBUG, _T("MySocket%d: lost socket."), m_ch);
	Disconnect();
}

void MySocket::OnConnectEvent(wxSocketEvent& event)
{
	// client connected
	emu->socket_connected(m_ch);
	// and writeable
	emu->socket_writeable(m_ch);
	// send data
	emu->send_data(m_ch);

	wxSocketBase *sock = event.GetSocket();

	// setup receive/lost event
	sock->SetEventHandler(*this, IDS_SOCKET_ID);
	sock->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG);
	sock->Notify(true);
}

//bool MySocket::Connect(const wxString &hostname, const wxString &service)
//{
//	wxString service = wxString::Format(wxT("%d"),port);
//	return Connect(hostname, service);
//}
bool MySocket::Connect(const wxString &hostname, int port)
{
	MyIPaddress addr;
	addr.Hostname(hostname);
	addr.Service(port);

	if (!((wxSocketClient *)mSocket)->Connect(addr, false)) {
		if (mSocket->LastError() != wxSOCKET_WOULDBLOCK) {
			return false;
		}
	}

	// connection ok
	mSocket->SetEventHandler(*this, IDS_CONNECT_ID);
	mSocket->SetNotify(wxSOCKET_CONNECTION_FLAG);
	mSocket->Notify(true);

	logging->out_logf(LOG_DEBUG, _T("MySocket%d: Client connection to %ls:%d"), m_ch, hostname.wc_str(), port);

	return true;
}
void MySocket::Disconnect()
{
	// disconnect
	emu->socket_disconnected(m_ch);
	conn->OnDestroy(m_ch);
}
size_t MySocket::Send(const char *buffer, size_t len)
{
	mSocket->Write(buffer, (wxUint32)len);
	len = mSocket->LastWriteCount();
	if (mSocket->Error()) {
		// error
		logging->out_logf(LOG_DEBUG, _T("MySocket%d: Socket write error: %d"),
				m_ch, mSocket->LastError());
	}
	return len;
}
size_t MySocket::Recv(char *buffer, size_t len)
{
	mSocket->Read(buffer, (wxUint32)len);
	len = mSocket->LastReadCount();
	if (mSocket->Error()) {
		// error
		logging->out_logf(LOG_DEBUG, _T("MySocket%d: Socket read error: %d"),
				m_ch, mSocket->LastError());
	}
	return len;
}
size_t MySocket::SendTo(const wxString &hostname, int port, const void *buffer, size_t len)
{
	if (!m_tcp) return 0;

	MyIPaddress addr;
	addr.Hostname(hostname);
	addr.Service(port);

	((wxDatagramSocket *)mSocket)->SendTo(addr, buffer, (wxUint32)len);
	len = mSocket->LastWriteCount();
	if (mSocket->Error()) {
		// error
		logging->out_logf(LOG_DEBUG, _T("MySocket%d: Socket write error: %d"),
				m_ch, mSocket->LastError());
	}
	return len;
}
size_t MySocket::RecvFrom(char *ipaddr, int *port, void *buffer, size_t len)
{
	mSocket->Read(buffer, (wxUint32)len);
	len = mSocket->LastReadCount();
	if (mSocket->Error()) {
		// error
		logging->out_logf(LOG_DEBUG, _T("MySocket%d: Socket read error: %d"),
				m_ch, mSocket->LastError());
	}
	MyIPaddress addr;
	mSocket->GetPeer(addr);
	if (ipaddr) {
		strcpy(ipaddr, addr.IPAddress().To8BitData());
	}
	if (port) *port = addr.Service();

	return len;
}
