/** @file wxw_socket.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.09.01

	@brief [ wxw socket ]
*/

#ifndef WXW_SOCKET_H
#define WXW_SOCKET_H

#include <wx/socket.h>

class EMU;
class MySocket;

/**
 * network
 */
class MyConnection : public wxObject
{
private:
	EMU *emu;
	MySocket **mSocket;
	bool *mSocketUsed;
	int mSocketMax;
	int mSocketNum;

public:
	MyConnection(EMU *parent, int max_sockets);
	~MyConnection();
	bool Connect(int new_ch, const _TCHAR *hostname, int port, bool server = false, bool tcp = true);
	bool Connect(const _TCHAR *hostname, int port, bool server = false, bool tcp = true);
	bool AcceptSocket(int ch, wxSocketBase *sock, bool tcp);
	int  GetSocketChannel();
	void Disconnect(int ch);
	size_t Send(int ch, const char *buffer, size_t len);
	size_t Recv(int ch, char *buffer, size_t len);
	size_t SendTo(int ch, const _TCHAR *hostname, int port, const void *buffer, size_t len);
	size_t RecvFrom(int ch, char *ipaddr, int *port, void *buffer, size_t len);

	void OnDestroy(int ch);

	EMU *GetEMU() { return emu; }
};

class MySocket : public wxEvtHandler
{
private:
	MyConnection *conn;
	EMU *emu;
	int m_ch;
	bool m_server;
	wxSocketBase *mSocket;
	bool m_ok;
	bool m_tcp;

	wxTimer mTimer;
	int mTimerType;

//	bool m_busy;

public:
	MySocket(MyConnection *parent, int ch, const wxString &hostname, int port, bool server = false, bool tcp = true);
//	MySocket(MyConnection *parent, int ch, const wxString &hostname, const wxString &service, bool server);
	MySocket(MyConnection *parent, int ch, wxSocketBase *sock);
	~MySocket();

	// event procedures
	void OnAcceptEvent(wxSocketEvent &);
	void OnConnectEvent(wxSocketEvent &);
	void OnSocketEvent(wxSocketEvent &);
	void OnSocketInputEvent(wxSocketEvent &);
	void OnSocketLostEvent(wxSocketEvent &);
	void OnTimerEvent(wxTimerEvent &);

	// properties
	bool IsOk() { return m_ok; }

	bool Connect(const wxString &hostname, int port);
//	bool Connect(const wxString &hostname, const wxString &service);
	void Disconnect();
	size_t Send(const char *buffer, size_t len);
	size_t Recv(char *buffer, size_t len);
	size_t SendTo(const wxString &hostname, int port, const void *buffer, size_t len);
	size_t RecvFrom(char *ipaddr, int *port, void *buffer, size_t len);

	enum {
		IDS_ACCEPT_ID = 10,
		IDS_CONNECT_ID,
		IDS_SOCKET_ID,
		IDS_TIMER_ID,

		IDT_CONNECTED,
	};

	DECLARE_EVENT_TABLE()
};

#endif /* WXW_SOCKET_H */
